/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Bath 
**
** DESCRIPTION:	Bath Restoration Puzzle
**
**      AUTHOR:	Arthur Humphrey, from template of Adrian Francis
**
\******************************************************************************/

#ifndef BUILDINGTOTEMPUZZLE_H
#define BUILDINGTOTEMPUZZLE_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwImage.h"
#include "PuzzleInterface.h"
#include "../Game/theGameState.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

//const ldwPoint Fruitwells_c_sCenter(114, 1802);

enum EBuildingTotemStatus
{
	eBuildingTotem_InitialState = 0,
	eBuildingTotem_WorkStarted,
	eBuildingTotem_OneQuarterDown,
	eBuildingTotem_OneHalfDown,
	eBuildingTotem_ThreeQuartersDown,
	eBuildingTotem_TotemDown,
	eBuildingTotem_NumberOfStates

};

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CBuildingTotemPuzzle : public IPuzzle
{
	friend class CPuzzle;

public:

	CBuildingTotemPuzzle(void);



	EBuildingTotemStatus m_eBuildingTotemStatus;

	void Reset();

	const int	LoadState(const unsigned char *pData);
	const int	SaveState(const unsigned char *pData, const int iMaxDataSize) const;
	bool		AdvancePuzzleState();
	EBuildingTotemStatus	 GetStatus();
	int GetPercentComplete();

	void Draw() const;
	

	void		Draw(const int iRef);
	const int SaveStateSize() {return sizeof (SSaveState);}
	void		LoadAssets();
	void		ReleaseAssets();

	// process
	void Refresh();
	void Update();
	void Advance(CVillager &Villager, const int iRef);
	void Advance();
	void AdvanceRebuild();
	void Complete();

	// access

private:
	

	//class structures
	struct SSaveState
	{
		EBuildingTotemStatus	m_eBuildingTotemStatus;
		int					m_iBuildingTotemProgress;
	};
	
	int m_iBuildingTotemProgress;

	ldwUINT32 m_uPlantTimer;
	//class data
	
	theGameState  *gs;

	ldwImageGrid *	m_pScaffold;

	ldwImageGrid *	m_pTotem;

};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CBuildingTotemPuzzle		BuildingTotemPuzzle;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/