/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Bath 
**
** DESCRIPTION:	Bath Restoration Puzzle
**
**      AUTHOR:	Arthur Humphrey, from template of Adrian Francis
**
\******************************************************************************/

#ifndef ClosedEyeTOTEMPUZZLE_H
#define ClosedEyeTOTEMPUZZLE_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwImage.h"
#include "PuzzleInterface.h"
#include "../Game/theGameState.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

//const ldwPoint Fruitwells_c_sCenter(114, 1802);

enum EClosedEyeTotemStatus
{
	eClosedEyeTotem_Initial = 0,
	eClosedEyeTotem_PickedUpPan,
	eClosedEyeTotem_PlacedPan,
	eClosedEyeTotem_Panning,
	eClosedEyeTotem_Smelting,
	eClosedEyeTotem_GoldInFire,
	eClosedEyeTotem_Installing,
	eClosedEyeTotem_Installed,

	eClosedEyeTotem_NumberOfStates

};

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CClosedEyeTotemPuzzle : public IPuzzle
{
	friend class CPuzzle;

public:

	CClosedEyeTotemPuzzle(void);



	EClosedEyeTotemStatus m_eClosedEyeTotemStatus;
	bool m_bPanInUse;
	bool m_bPotInUse;
	bool m_bBallInUse;

	ldwUINT32 m_uSmeltTimer;
	void Reset();

	const int	LoadState(const unsigned char *pData);
	const int	SaveState(const unsigned char *pData, const int iMaxDataSize) const;
	bool		AdvancePuzzleState();
	EClosedEyeTotemStatus	 GetStatus();
	int GetPercentComplete();
	int GetProgress();
	void StartPuzzle();
	void SetPanInUse(bool inUse);
	bool GetPanInUse();
	void Draw()const;
	

	void		Draw(const int iRef);
	const int SaveStateSize() {return sizeof (SSaveState);}
	void		LoadAssets();
	void		ReleaseAssets();

	// process
	void Refresh();
	void Update();
	void Advance(CVillager &Villager, const int iRef);
	void Advance();
	void Complete();

	// access

private:
	

	//class structures
	struct SSaveState
	{
		EClosedEyeTotemStatus	m_eClosedEyeTotemStatus;
		int					m_iClosedEyeTotemProgress;
	};
	
	int m_iClosedEyeTotemProgress;
	
	//class data
	float m_fCanopyAlpha;
	float m_fWispAlpha;
	float m_fWispFrameCount;
	bool  m_bWispFadeComplete;
	bool  m_bWispFadeOut;

	theGameState  *gs;

	ldwImageGrid *	m_pScaffold;

	ldwImageGrid *	m_pTotem;
	ldwImageGrid *	m_pPan;
	ldwImageGrid *	m_pSmeltPot;
	ldwImageGrid *	m_pGoldBall;
	ldwImageGrid *	m_pClosedEyeTotem;
	ldwImageGrid *	m_pCanopy;
	ldwImageGrid *  m_pWispFrame;

};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CClosedEyeTotemPuzzle		ClosedEyeTotemPuzzle;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/