/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Collectable
**
** DESCRIPTION:	Manages collectables - anything which can be picked up
**
**      AUTHOR:	Adrian Francis
**
\******************************************************************************/

#ifndef COLLECTABLE_H
#define COLLECTABLE_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
class CVillager;
class ICollectable;

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int Collectable_c_iMaxObservers = 3;

/******************************************************************************\
** CLASS ENUMERATIONS
\******************************************************************************/
enum ECarrying
{
	eCarrying_Nothing = -1,

	eCarrying_Berries,
	eCarrying_Rocks,
	eCarrying_Feathers,
	eCarrying_Baby,
	eCarrying_Wood,
	eCarrying_Laundry,
	eCarrying_Crops,
	eCarrying_Food = eCarrying_Crops,
	eCarrying_Flowers,
	eCarrying_Fish,
	eCarrying_Crab,
	eCarrying_Water,
	eCarrying_Body,
	eCarrying_MagicFish,
	eCarrying_Shroom,
	eCarrying_RareShroom,
	eCarrying_Kindling,
	eCarrying_Coconut,
	eCarrying_EmptyBowl,
	eCarrying_Tool,
	eCarrying_Pot,
	eCarrying_FullPot,
	eCarrying_BananaLeaf,
	eCarrying_Speartips,
	eCarrying_Bounty,
	eCarrying_BlueBowl,
	eCarrying_Rope,

	eCarrying_AlchemyRelic1,
	eCarrying_AlchemyRelic2,
	eCarrying_AlchemyRelic3,
	eCarrying_AlchemyRelic4,
	eCarrying_AlchemyRelic5,
	eCarrying_Herb1,
	eCarrying_Herb2,
	eCarrying_Herb3,
	eCarrying_Herb4,
	eCarrying_Herb5,
	eCarrying_Herb6,
	eCarrying_Herb7,
	eCarrying_Banana,
	eCarrying_Mango,
	eCarrying_Papaya,
	eCarrying_Honey,
	eCarrying_Pulp = eCarrying_Honey,
	eCarrying_SmokingStick,
    eCarrying_BowlOfJunk,
	eCarrying_Corkrock,
	eCarrying_Key2,
	eCarrying_Key3,
	eCarrying_SharkCharmerVial,
	eCarrying_Leaves,
	eCarrying_GiantPearl,
	eCarrying_BurnedTorch,
	eCarrying_BrokenPearl,
	eCarrying_HotRock,
	eCarrying_Cloth,
	eCarrying_FruitBreadYellow,
	eCarrying_FruitBreadOrange,
	eCarrying_FlyingFish,
	eCarrying_FlyingFishGold,
	eCarrying_Soap,
	eCarrying_Fishbone,
	eCarrying_FishboneBurnt,
	eCarrying_BlackBeetle,
	eCarrying_BlackBerriesLG,
	eCarrying_MothEatenRope,
	eCarrying_FinishedLei,
	eCarrying_LeiFlower1,
	eCarrying_LeiFlower2,
	eCarrying_LeiFlower3,
	eCarrying_Frog,
	eCarrying_BraidsInCrate,
	eCarrying_NecklacePieceScientist,
	eCarrying_NecklacePieceFarmer,
	eCarrying_NecklacePieceDoctor,
	eCarrying_NecklacePieceBuilder,
	eCarrying_Necklace,
	eCarrying_DyePotRed,
	eCarrying_DyePotYellow,
	eCarrying_DyePotBlue,
	eCarrying_FullSmeltPot,
	eCarrying_GoldEye,
	// Items below are not part of the carrying image strip!
	eCarrying_BeginCollectables,

	eCarrying_BeginCollectableScienceItems = eCarrying_BeginCollectables,

	eCarrying_ScienceItem1 = eCarrying_BeginCollectableScienceItems,
	eCarrying_ScienceItem2,
	eCarrying_ScienceItem3,
	eCarrying_ScienceItem4,
	eCarrying_ScienceItem5,
	eCarrying_ScienceItem6,
	eCarrying_ScienceItem7,
	eCarrying_ScienceItem8,

	eCarrying_ScienceItem9,
	eCarrying_ScienceItem10,
	eCarrying_ScienceItem11,
	eCarrying_ScienceItem12,
	eCarrying_ScienceItem13,
	eCarrying_ScienceItem14,
	eCarrying_ScienceItem15,
	eCarrying_ScienceItem16,

	eCarrying_ScienceItem17,
	eCarrying_ScienceItem18,
	eCarrying_ScienceItem19,
	eCarrying_ScienceItem20,
	eCarrying_ScienceItem21,
	eCarrying_ScienceItem22,
	eCarrying_ScienceItem23,
	eCarrying_ScienceItem24,

	eCarrying_EndCollectableScienceItems = eCarrying_ScienceItem24,

	eCarrying_BeginCollectableRelics,

	eCarrying_Relic1                 = eCarrying_BeginCollectableRelics,
	eCarrying_Relic2,
	eCarrying_Relic3,
	eCarrying_Relic4,
	eCarrying_Relic5,
	eCarrying_Relic6,
	eCarrying_Relic7,
	eCarrying_Relic8,

	eCarrying_Relic9,
	eCarrying_Relic10,
	eCarrying_Relic11,
	eCarrying_Relic12,
	eCarrying_Relic13,
	eCarrying_Relic14,
	eCarrying_Relic15,
	eCarrying_Relic16,

	eCarrying_Relic17,
	eCarrying_Relic18,
	eCarrying_Relic19,
	eCarrying_Relic20,
	eCarrying_Relic21,
	eCarrying_Relic22,
	eCarrying_Relic23,
	eCarrying_Relic24,

	eCarrying_EndCollectableRelics = eCarrying_Relic24,

	eCarrying_Shroom1,
	eCarrying_Shroom2,
	eCarrying_Shroom3,
	eCarrying_NumberOf
};

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CCollectable
{
private:

	//class structures
	struct SCollectable
	{
		ICollectable *	m_pObserver[Collectable_c_iMaxObservers];
		int				m_iNumObservers;
	};

	//class data
//	static ICollectable *	sm_pCollectable[eCarrying_NumberOf];

	SCollectable		m_sCollectable[eCarrying_NumberOf];

	//class methods
	//setup
	public:		CCollectable(void);
	public:		void RegisterObserver(const ECarrying eCarrying, ICollectable *pObserver);

	// process
	public:		const bool ProcessNearbyCollectables(CVillager &Villager) const;
	public:		const bool Carry(CVillager &Villager, const ECarrying eCarrying) const;
	public:		void Drop(CVillager &Villager, const ECarrying eCarrying) const;

	// access
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CCollectable		Collectable;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/