/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	CollectionScene
**
** DESCRIPTION:	Collection scene displays items which have been collected
**
**
\******************************************************************************/

#ifndef THE_COLLECTION_SCENE_H
#define THE_COLLECTION_SCENE_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwScene.h"
#include "ldwButton.h"
#include "ldwTextControl.h"

#include "../Collectables/Collectable.h"
#include "../Utils/theGraphicsManager.h"
#include "../Utils/theStringManager.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int CollectionScene_c_iNumCollectables = (1 + eCarrying_EndCollectableRelics - eCarrying_BeginCollectables) + 4/*necklace pieces*/;

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CCollectionScene : public ldwScene
{
public:

	static CCollectionScene *	m_pInstance;

	static CCollectionScene *Get();
	static CCollectionScene &Ref();
	static void Destroy();

	CCollectionScene(const CCollectionScene &);
	CCollectionScene & operator = (const CCollectionScene &);

	bool HandleKey(int iMessageID, int iKeyCode, int iCharacter);
	bool HandleMessage(int iMessageID, long iData);
	bool HandleMouse(int iMessageID, ldwPoint sMousePos);


protected:
	
	void Activate(bool bIsActive);
	void UpdateScene();
	void DrawScene();

private:

	CCollectionScene();
	~CCollectionScene();

	//class enumerations
	enum EButton
	{
		eButton_Done = 0,

		eButton_NumberOf,
	};

	enum ECollection
	{
		eCollection_ScienceItems = 0,
		eCollection_Relics,
		eCollection_Necklace,

		eCollection_NumberOf
	};

	//class structures
	struct SCollectable
	{
		EImage				m_eImage;
		int					m_iXPos;
		int					m_iYPos;
	};

	struct SCollection
	{
		StringId			m_eLabel;
		ldwPoint			m_sLabelPos;
	};

	static SCollectable		sm_sCollectable[CollectionScene_c_iNumCollectables];
	static SCollection		sm_sCollection[eCollection_NumberOf];

	int						m_iCursorOnCollectable;

	ldwButton *				m_pButton[eButton_NumberOf];
	ldwTextControl *		m_pSceneLabel;
	ldwTextControl *		m_pPlayerCommField;
	ldwTextControl *		m_pCollectionLabel[eCollection_NumberOf];
	ldwTextControl *		m_pCollectionProgress[eCollection_NumberOf];
};

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/