/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	ContentMap
**
** DESCRIPTION:	Maintains the content of the world map, including objects,
**				hotspots, collectables, and blocking information.
**
**
\******************************************************************************/

#ifndef CONTENTMAP_H
#define CONTENTMAP_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwTypes.h"
#include "../Game/Isola.h"	//EDirection

#include "WorldMap.h"

class CBehaviorEditor;
class CContentMapEditor;
class CWorldPath;

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

// Block width and height MUST be a power of 2!

const int ContentMap_c_iBlockWidth = 8;
const int ContentMap_c_iBlockHeight = 8;

const int ContentMap_c_iWidthInBlocks  = (WorldMap_c_iWidthInBlocks  * WorldMap_c_iBlockWidth) / ContentMap_c_iBlockWidth;
const int ContentMap_c_iHeightInBlocks = (WorldMap_c_iHeightInBlocks * WorldMap_c_iBlockHeight) / ContentMap_c_iBlockHeight;

// A known legal position near the center of the map where something can be
// placed.  Used by FindLegalTerrain.
const ldwPoint ContentMap_c_sSnapToCenter = ldwPoint(795, 1500);

const int ContentMap_c_iMaxObjectCandidates = 1024;

enum ETerritory
{
	eTerritory_Any,
	eTerritory_Tribe1,
	eTerritory_Tribe2,

	eTerritory_NumberOf,

	eTerritory_HeathenTribe = eTerritory_Tribe2
};

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CContentMap
{
	friend class CBehaviorEditor;
	friend class CContentMapEditor;
	friend class CWorldPath;
	friend class CContentMapUtil;
	friend class CHouse;

public:

	//class enumerations
	enum EMaterial
	{
		eMaterial_None = 0,
		eMaterial_Soil,
		eMaterial_Grass,
		eMaterial_Sand,
		eMaterial_ShallowWater,
		eMaterial_DeepWater,
		eMaterial_Rock,
		eMaterial_Interior,

		eMaterial_NumberOf
	};

	enum EObject
	{
		eObject_None = 0,
		eObject_Fireplace,
		eObject_Hut,
		eObject_ScienceHut,
		eObject_Firewood,
		eObject_Foodbin,
		eObject_Fire,
		eObject_House1Site,
		eObject_House2Site,
		eObject_House3Site,
		eObject_House1,
		eObject_House2,
		eObject_House3,
		eObject_ResearchTable,
		eObject_ResearchEquipment,
		eObject_VillageCenter,
		eObject_ClothingHutSite,
		eObject_LoveShackSite,
		eObject_ClothingHut,
		eObject_LoveShack,
		eObject_RosterOfTheDead,
		eObject_House1Door,
		eObject_House2Door,
		eObject_House3Door,
		eObject_ClothingHutDoor,
		eObject_LoveShackDoor,
		eObject_BlackberryBush,
		eObject_Kindling,
		eObject_Puddle,
		eObject_HospitalHutSite,
		eObject_HospitalHut,
		eObject_HospitalHutDoor,
		eObject_NurseryHutSite,
		eObject_NurseryHut,
		eObject_NurseryHutDoor,
		eObject_LabEquipment,
		eObject_HospitalLabTable1,
		eObject_HospitalBed1,
		eObject_StoryTeller,
		eObject_StoryListener,
		eObject_FenceDeconstruct,
		eObject_GranaryBuild,
		eObject_TownCenter,
		eObject_FoodbinBlocking,
		eObject_FenceBlockingN,
		eObject_FenceBlockingE,
		eObject_FenceBlockingS,
		eObject_FenceBlockingW,
		eObject_WaterBlockage,
		eObject_Food1Totem,
		eObject_ResearchTotem,
		eObject_NurseryTotem,
		eObject_MedicineTotem,
		eObject_BuildingTotem,
		eObject_BelieverIdol,
		eObject_HeathenBathingArea,
		eObject_RestoreMausoleum,
		eObject_HeathenCenter,
		eObject_FoodTotemBlocking,
		eObject_ResearchTotemBlocking,
		eObject_MedicineTotemBlocking,
		eObject_BuildingTotemBlocking,
		eObject_GroveLocation,
		eObject_LightningHidingPlace,
		eObject_PolishIdolLevel2,
		eObject_BuildIdolSupplyArea1,
		eObject_CrumblyRock,
		eObject_Childrens1Totem,
		eObject_HeathenBathroom,
		eObject_BelieverPrayerArea,
		eObject_YellowDyeBucket,
		eObject_RedDyeBucket,
		eObject_BlueDyeBucket,
		eObject_RainbowTotem,
		eObject_DyeDeploymentArea,
		eObject_NervousPoolPacing,
		eObject_StdRunAwayLocations,
		eObject_FoodPatrolPoint1,
		eObject_FoodPatrolPoint2,
		eObject_LabPatrolPoint1,
		eObject_LabPatrolPoint2,
		eObject_SharedChildrenPlay,
		eObject_HydroFarm,
		eObject_Chalkboard,
		eObject_HeathenScientistHangout,
		eObject_GroveCenter,
		eObject_ChildrensTotemBlocking,
		eObject_HeathenBuilderHangout,
		eObject_ChalkboardClose,
		eObject_ChalkboardCloseRt,
		eObject_CorpseCenter,
		eObject_builderChallengeLt,
		eObject_builderChallengeRt,
		eObject_Aqueduct,
		eObject_DirectAqueductRepairNE,
		eObject_DirectAqueductRepairNW,
		eObject_ClosedEyePanLocation,
		eObject_ClosedEyeSmeltPotLocation,
		eObject_HiddenPanLocation,
		eObject_ClosedEyeTotemLocation,
		eObject_FarmerNecklacePieceStart,
		eObject_ScientistNecklacePieceStart,
		eObject_DoctorNecklacePieceStart,
		eObject_BuilderNecklacePieceStart,
		eObject_NecklacePieces,
		eObject_Lake,
		eObject_Puddles,
		eObject_FishingLocation,
		eObject_HeathenChiefLocation,
		eObject_NecklaceFinalDestination,
		eObject_BelieverPrayerAreaWest,
		eObject_MotherHeathenHidingPlace,
		eObject_BelieverMotherTalkingPos,
		eObject_EyeTotemBlocking,
		eObject_RainbowTotemBlocking,
		eObject_DrinkingLocation,
		eObject_EnjoyLakeLocation,
		eObject_QuietLocation,
		eObject_DanceLocation,
		eObject_LabEntrance,
		eObject_Steps,
		eObject_RainDanceLocation,
		eObject_HonorDeadLocation1,
		eObject_HonorDeadLocation2,
		eObject_Pool,
		eObject_HeathenGuardBathingArea,
		eObject_WestDivingPlatform,
		eObject_Bathroom,
		eObject_PlayTag,
		eObject_ChangeBaby,
		eObject_WashBaby,
		eObject_GroveEntrance,
		eObject_NorthRiver_East,
		eObject_SouthRiver_East,
		eObject_MudHutRepairArea,

		eObject_NumberOf
	};

	enum EHotSpot
	{
		eHotSpot_None = 0,
		eHotSpot_Fireplace,
		eHotSpot_Firewood,
		eHotSpot_Kindling,
		eHotSpot_Foodbin,
		eHotSpot_Research,
		eHotSpot_House1Site,
		eHotSpot_House2Site,
		eHotSpot_House3Site,
		eHotSpot_ClothingHutSite,
		eHotSpot_LoveShackSite,
		eHotSpot_LoveShack,
		eHotSpot_RosterOfDead,
		eHotSpot_HospitalLab,
		eHotSpot_BlackberryBush,
		eHotSpot_Puddle,
		eHotSpot_HospitalHutSite,
		eHotSpot_NurseryHutSite,
		eHotSpot_FenceDeconstruct,
		eHotSpot_Food1Totem,
		eHotSpot_ResearchTotem,
		eHotSpot_NurseryTotem,
		eHotSpot_MedicineTotem,
		eHotSpot_BuildingTotem,
		eHotSpot_BelieverIdol,
		eHotSpot_RestoreMausoleum,
		eHotSpot_ChildrensTotem,
		eHotSpot_YellowDyeBucket,
		eHotSpot_RedDyeBucket,
		eHotSpot_BlueDyeBucket,
		eHotSpot_RainbowTotem,
		eHotSpot_Chalkboard,
		eHotSpot_BuilderChallengeLt,
		eHotSpot_BuilderChallengeRt,
		eHotSpot_HydroFarm,
		eHotSpot_Aqueduct,
		eHotSpot_ClosedEyePanLocation,
		eHotSpot_HiddenPanLocation,
		eHotSpot_FarmerNecklacePiece,
		eHotSpot_ScientistNecklacePiece,
		eHotSpot_DoctorNecklacePiece,
		eHotSpot_BuilderNecklacePiece,
		eHotSpot_NecklacePieces,
		eHotSpot_Lake,
		eHotSpot_HeathenChief,
		eHotSpot_ClosedEyeTotemLocation,
		eHotSpot_WestDivingPlatform,
		eHotSpot_MidGameMask,
		eHotSpot_Pools,
		//eHotSpot_HospitalPracticeArea,

		eHotSpot_NumberOf
	};

	enum ERedirector
	{
		eRedirector_None = 0,
		eRedirector_DivingRock,

		eRedirector_NumberOf
	};

	struct SContent
	{
		bool			m_bIsBlocking;
		EMaterial		m_eMaterial;
		EObject			m_eObject;
		EHotSpot		m_eHotSpot;
		ERedirector		m_eRedirector;
		ETerritory      m_territory;

		SContent()
		{
		}

		SContent(const bool bIsBlocking, const EMaterial eMaterial, const EObject eObject, const EHotSpot eHotSpot, const ERedirector eRedirector, const ETerritory eTerritory = eTerritory_Any)
		{
			m_bIsBlocking = bIsBlocking;
			m_eMaterial = eMaterial;
			m_eObject = eObject;
			m_eHotSpot = eHotSpot;
			m_eRedirector = eRedirector;
			m_territory = eTerritory;
		}
	};

	//setup
	CContentMap(void);

	// process

	void Load();
	
	const ldwPoint FindObject(const EObject eObject,ETerritory territory = eTerritory_Any,bool blockedOK = false);

	bool IsObjectInArea(const EObject eObject,ldwRect targetArea);
	 
	const ldwPoint FindLegalTerrain(ldwPoint searchCenter,int xTolerance,int yTolerance,ETerritory territory = eTerritory_Any,const bool bIsDeepWaterLegal = true,const bool bIsObjectLegal = true) const;
	const ldwPoint FindLegalTerrain(ldwPoint sStartPos,EDirection directionalBias,ETerritory territory = eTerritory_Any, const bool bIsDeepWaterLegal = true,const bool bIsObjectLegal = true) const;

	ldwPoint FindTribe1LegalTerrain();

	// access

	const bool 		  IsBlocking(const ldwPoint sPos,ETerritory territory = eTerritory_Any) const;
	const EMaterial   GetMaterial(const ldwPoint sPos) const;
	const EObject 	  GetObject(const ldwPoint sPos) const;
	const EHotSpot 	  GetHotSpot(const ldwPoint sPos) const;
	const ERedirector GetRedirector(const ldwPoint sPos) const;

	void			  SetObject(const ldwPoint sPos,EObject object,bool blocked = false);

	void SetObjectBlocking(const EObject eObject, const bool bIsBlocking);
	void SetObjectMaterial(const EObject eObject, const EMaterial eMaterial);
	void SetObjectHotSpot(const EObject eObject, const EHotSpot eHotSpot);
	void SetObjectRedirector(const EObject eObject, const ERedirector eRedirector);
	void SetObjectContent(const EObject eObject, const SContent &sNewContent, const bool bSetBlocking, const bool bSetMaterial, const bool bSetHotSpot, const bool bSetRedirector);

	const bool IsValidSpawnPos(const ldwPoint sPos) const;

	ldwPoint WorldPointToMapPoint(ldwPoint worldPoint) {return ldwPoint(worldPoint.x/ContentMap_c_iBlockWidth ,worldPoint.y/ContentMap_c_iBlockHeight );}
	const ldwPoint GetRandomPos(const EMaterial material = eMaterial_None, const bool bCheckBlocking = false);
	const ldwPoint GetRandomSpawnPos(const EMaterial material = eMaterial_None);
	const ldwPoint GetRandomSpawnPos(const ldwRect &targetArea);

	void HighLightTribe1Area();

	ETerritory GetTerritory(ldwPoint where);

	void ResetRegion(const EObject eObject);

protected:

	void SetContent(const ldwRect sRect, const bool bIsBlocking, const EMaterial eMaterial = eMaterial_None, const EObject eObject = eObject_None, const EHotSpot eHotSpot = eHotSpot_None, const ERedirector eRedirector = eRedirector_None);
	void SetContent(const ldwRect sRect, const SContent &sContent);
	void SetContent(const ldwRect sRect, const SContent &sNewContent, const bool bSetBlocking, const bool bSetMaterial, const bool bSetObject, const bool bSetHotSpot, const bool bSetRedirector, const SContent &sMatchContent = SContent(false, eMaterial_None, eObject_None, eHotSpot_None, eRedirector_None));
	void GetContent(const ldwPoint sPos, SContent &sContent) const;

	inline const bool 	  	 ReadBlocking(const int iRow, const int iColumn) const;
	inline const EMaterial 	 ReadMaterial(const int iRow, const int iColumn) const;
	inline const EObject   	 ReadObject(const int iRow, const int iColumn) const;
	inline const EHotSpot  	 ReadHotSpot(const int iRow, const int iColumn) const;
	inline const ERedirector ReadRedirector(const int iRow, const int iColumn) const;

	bool IsCorrectTerritory(int iRow,int iColumn,ETerritory territory) const;
	
	void Draw(const bool bShowBlocking, const bool bShowMaterials, const bool bShowObjects, const bool bShowHotSpots, const bool bShowRedirectors, const float fAlpha, const SContent *psHighlightedContent = 0) const;


private:

	void Write(const int iRow, const int iColumn, const SContent &sContent);
	void Read(int iRow, int iColumn, SContent &sContent) const;
	void ConvertPosToRowColumn(const ldwPoint sPos, int &iRow, int &iColumn) const;
	void ConvertRowColumnToPos(const int iRow, const int iColumn, ldwPoint &sPos) const;

	//class data

#ifdef VV5_LARGE_MAP
#ifdef _DEBUG
	unsigned int	m_uiSmallMap[256][256];
#endif
#endif
	unsigned int	m_uiMap[ContentMap_c_iHeightInBlocks][ContentMap_c_iWidthInBlocks];

	struct sMapRegion
	{
		ldwRect	regionRect;
		bool    needsUpdate;
	};

	sMapRegion  m_objectRegion[eObject_NumberOf];

#if _DEBUG
	const bool Save();

	unsigned int	m_uiMapUndoData[ContentMap_c_iHeightInBlocks][ContentMap_c_iWidthInBlocks];

	bool	m_bUndoAvailable;
	bool    m_bRedoAvailable;

	bool	CanUndo() {return m_bUndoAvailable;}
	bool	CanRedo() {return m_bRedoAvailable;}

	void	Undo();
	void	SaveForUndo();

	void 	ResetRegions();

#endif
};

#include <assert.h>

inline const CContentMap::EMaterial CContentMap::ReadMaterial(const int iRow, const int iColumn) const
{
		assert(iRow >= 0 && iRow < ContentMap_c_iHeightInBlocks &&
		iColumn >= 0 && iColumn < ContentMap_c_iWidthInBlocks);

		return (EMaterial)((m_uiMap[iRow][iColumn] & 0x0000000E) >> 1);
}

inline const bool CContentMap::ReadBlocking(const int iRow, const int iColumn) const
{
	assert(iRow >= 0 && iRow < ContentMap_c_iHeightInBlocks &&
		iColumn >= 0 && iColumn < ContentMap_c_iWidthInBlocks);

	return (m_uiMap[iRow][iColumn] & 0x00000001) ? true : false;
}

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CContentMap		ContentMap;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/