/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	ContentMapEditor
**
** DESCRIPTION:	Editor for the content map
**
**
\******************************************************************************/

#ifndef CONTENTMAP_EDITOR_H
#define CONTENTMAP_EDITOR_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwTypes.h"
#include "ldwDropDown.h"
#include "EditorInterface.h"
#include "../World/ContentMap.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CContentMapEditor : public IEditor
{
public:

	CContentMapEditor(void);
	~CContentMapEditor(void);
	void Reset();

	// process
	void Show(const bool bShow);
	void Draw();
	const bool HandleKeyCharacter(const char cKey);
	const bool HandleKeyDown(const int iKeyCode);
	const bool HandleKeyUp(const int iKeyCode);
	const bool HandleMouseDown(const ldwPoint sMousePos);
	const bool HandleMouseUp(const ldwPoint sMousePos);
	const bool HandleMouseMove(const ldwPoint sMousePos);

	virtual bool ShowDecals() {return m_bShowDecals;}

	// access					
	const char *GetMaterialName(const CContentMap::EMaterial eMaterial) const;
	const char *GetObjectName(const CContentMap::EObject eObject) const;
	const char *GetObjectEnumName(const CContentMap::EObject eObject) const;
	const char *GetHotSpotName(const CContentMap::EHotSpot eHotSpot) const;
	const char *GetRedirectorName(const CContentMap::ERedirector eRedirector) const;


private:
	//class enumerations
	enum EMode
	{
		eMode_Select = 0,
		eMode_Edit,
		eMode_EditLayer,
		eMode_Erase,

		eMode_NumberOf
	};

	enum EDropDown
	{
		eDropDown_Material = 0,
		eDropDown_Object,
		eDropDown_HotSpot,
		eDropDown_Redirector,

		eDropDown_NumberOf
	};

	enum {ePolyPointsLimit = 1000, eWalkingColorCount = 3};

	bool m_bMoveMenu;

	//class data
	bool						m_bShowBlocking;
	bool						m_bShowMaterials;
	bool						m_bShowObjects;
	bool						m_bShowHotSpots;
	bool						m_bShowRedirectors;
	bool						m_bShowDecals;

	bool						m_bSelectionIsBlocking;

	ldwDropDown *				m_pDropDown[eDropDown_NumberOf];
	int							m_iTopDropDown;

	float						m_fContentMapAlpha;

	bool						m_bIsMouseDown;
	bool						m_bIsDragging;
	bool						m_bIsRectSelectionEnabled;
	ldwRect						m_sRect;
	ldwPoint					m_DragStart;

	bool						m_bPolyModeEnabled;
	bool						m_bInPolyMode;
	ldwPoint					m_PolyPoints[ePolyPointsLimit + 1];
	int							m_PolyPointCount;
	ldwPoint					m_lastPolyPos;

	EMode						m_eMode;

	static ldwColor	m_walkingColors[eWalkingColorCount];

	int 	m_walkingColorIndex;

	const char *GetModeName(const EMode eMode) const;

	void FillPoly();
	void DrawPolyOutline(bool setContent = false);
	void DrawPolyLine(ldwPoint startPt,ldwPoint endPt,bool firstLine,bool setContent = false);
	void DrawBlock(int x,int y,ldwColor color,bool setContent = false);

	ldwColor NextWalkingColor();

	ldwPoint ScreenToContentMapCoordinates(ldwPoint screenPos);
};


/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CContentMapEditor		ContentMapEditor;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/