//
//		Definition of the main game class
//
#ifndef THE_DECAL_MANAGER_H
#define THE_DECAL_MANAGER_H


#include "../World/WorldMap.h"
#include "../Utils/theGraphicsManager.h"
#include "../World/Environment.h"

#include "ldwScene.h"
#include "ldwGameWindow.h"
#include "ldwImage.h"

class ldwButton;
class ldwSoundTrack;
class ldwSoundFx;
class theGameState;
class ldwTextControl;


//...typedefs etc

#define maxNumDecals  256
#define maxNumWaves 9

#define hut1DecalX						480
#define hut1DecalY						576
#define hut2DecalX						369
#define hut2DecalY						805
#define hut3DecalX						448
#define hut3DecalY						1015
#define hospitalDecalX					665
#define hospitalDecalY					1246
#define hospitalFoundation2DecalX		775
#define hospitalFoundation2DecalY		1255

typedef struct{

	bool 			exists;
	ldwPoint		m_sPos;
	ldwPoint		m_sOffset;
	ldwPoint		m_sOffsetMax;
	int				m_offsetBump;
	int 			waveVel;
	double 			angle;
	double 			angleStep;
	float 			waveOpacity;
	ldwImageGrid* 	waveGraphic;

} OceanWavesType;

typedef struct{

	bool          exists;
	ldwPoint      decalPosition;
	ldwImageGrid *decalImage;
	int           column;

} DecalsType;
//END DEFINES

//BEGIN CLASS DECLARATION
class CDecal
{
	//class methods
	//setup
	public:		CDecal();
	public:		void Reset();
	public:		void LoadAssets();
	public:		void ReleaseAssets();

	DecalsType decal[maxNumDecals];
	void InitDecals();
	void RefreshDecals();
	void RefreshProps();
	void AddDecal(EImage imageId,int x, int y, int column = 0);
	void AddDecal(ldwImageGrid *whatImage, int x, int y, int column = 0);
	void DrawDecals();
	void DrawArrow();
	void DrawWaves();
	void DrawMiniDecals(int baseX,int baseY,float scale);

	void ShowArrow(ldwImage *arrow,int x,int y);
	void HideArrow();

private:

	OceanWavesType waves[maxNumWaves + 1];
    	
	ldwGameWindow *gameWindow;
	theGameState  *gs;

	ldwImage *mArrow;
	ldwPoint  mArrowPos;
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CDecal		Decal;

#endif