  /******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Floating Anim
**
** DESCRIPTION:	Manages floating anims
**
**      AUTHOR:	Adrian Francis
**
\******************************************************************************/
#ifndef FLOATING_ANIM_H
#define FLOATING_ANIM_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwImage.h"
#include "../Utils/SceneManager.h"
#include "../Utils/theGraphicsManager.h"
#include "../Villagers/VillagerAttachmentInterface.h"
#include "ldwGameWindow.h"
/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int FloatingAnim_c_iMaxFloatingAnims = 512;
const int FloatingAnim_c_iNumFramesBetweenUpdates = 3;

/******************************************************************************\
** CLASS ENUMERATIONS
\******************************************************************************/
enum EFloatingAnim
{
	eFloatingAnim_None = -1,

	eFloatingAnim_Waves1 = 0,
	eFloatingAnim_SparkleYellow,
	eFloatingAnim_WhiteSparkle,
	eFloatingAnim_WhiteSparkleBig,
	eFloatingAnim_Dust,
	eFloatingAnim_Algae,
	eFloatingAnim_Rainbow,
	eFloatingAnim_Bubble,
	eFloatingAnim_Candle,
	eFloatingAnim_Fire,
	eFloatingAnim_Petal,
	eFloatingAnim_WaterfallFoam,
	eFloatingAnim_ItemDrop,
	eFloatingAnim_Spray,
	eFloatingAnim_SharpSparkle,
	eFloatingAnim_BlurrySparkle,
	eFloatingAnim_CollectableDrop,
	eFloatingAnim_Footprint,
	eFloatingAnim_RainDrop,
	eFloatingAnim_Smoke,
	eFloatingAnim_ReefFish1,
	eFloatingAnim_ReefFish2,
	eFloatingAnim_ReefFish3,
	eFloatingAnim_SwarmOfBees,
	eFloatingAnim_TinyBubble,
	eFloatingAnim_Frog,
	eFloatingAnim_Frog2,
	eFloatingAnim_DarkSmoke,
	eFloatingAnim_FlyingFish1,
	eFloatingAnim_FlyingFish2,
	eFloatingAnim_CrawlingCrab1,
	eFloatingAnim_CrawlingCrab2,
	eFloatingAnim_Leaf,
	eFloatingAnim_Pond,
	eFloatingAnim_Undertow1,
	eFloatingAnim_FishFlop,
	eFloatingAnim_teleportSparkles,
	eFloatingAnim_lightEffect,
	eFloatingAnim_RisingLightRays,
	eFloatingAnim_RockSlide,
	eFloatingAnim_RockSlide2,		//NOTE: Keep this below RockSlide - do not reorder
	eFloatingAnim_WilloWispSml,
	eFloatingAnim_blinkEyesMask,
	eFloatingAnim_blinkEyesMaskSm,
	eFloatingAnim_Willowisp1Bright,
	eFloatingAnim_SmallDust,
	eFloatingAnim_Waterfall1,
	eFloatingAnim_Waterfall2,
	eFloatingAnim_Waterfall3,
	eFloatingAnim_Waterfall4,

	eFloatingAnim_NumberOf
};	

enum EFloatingAnimEffect
{
	eFloatingAnimEffect_None = -1,

	eFloatingAnimEffect_Rising = 0,
	eFloatingAnimEffect_Falling,
	eFloatingAnimEffect_Launching,
	eFloatingAnimEffect_Vibrating,
	eFloatingAnimEffect_RisingFast,
	eFloatingAnimEffect_FadeOut,
	eFloatingAnimEffect_Wander,
	eFloatingAnimEffect_Floating,

	eFloatingAnimEffect_NumberOf
};	

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CFloatingAnim : public IVillagerAttachment
{
private:

	struct SAnim
	{
		union UImage
		{
			EImage				m_eImage;
			ldwImageGrid *		m_pImage;
		};

		UImage					m_uImage;
		int						m_iNumFrames;
		bool					m_bStartOnRandomFrame;
		int						m_iSortOffset;
	};

	struct SInst
	{
		friend class CFloatingAnim;

	private:

		//structure data
		EFloatingAnim			m_eType;
		EFloatingAnimEffect		m_eEffect;
		ldwPoint				m_sPos;
		bool					m_bIsCentered;
		int						m_iCurrentFrame;
		int						m_iNumFrames;
		int						m_iAnimSpeed;
		int						m_iAnimIdx;
		ldwPoint				m_sOffset;
		ldwPoint				m_sVel;
		int						m_iWhichItem;
		bool					m_bIsLooping;
		EScenePriority			m_eScenePriority;
		int						m_iEffectSpeed;
		ldwImageGrid *			m_pAnim;

		//structure methods
		public:		void UpdateOffset();
	};

	//class data
	static SAnim				m_sAnim[eFloatingAnim_NumberOf];
	SInst						m_InstArray[FloatingAnim_c_iMaxFloatingAnims];
	ldwGameWindow *  gameWindow;
	// from here is ligthing experiment stuff

	//int maxNodes;
	ldwUINT32 timer;
	//float fade;
	//to here (end lightning exp stuff)
	int							m_iFrameCounter;
	bool						m_bAllowAddWhilePaused;

	//class methods
	//setup
public:

	CFloatingAnim();
	~CFloatingAnim();
	void LoadAssets();
	void ReleaseAssets();
	void Reset();
	ldwUINT32 GetLightningTimer(){return timer;}
	void SetLightningTimer(ldwUINT32 setTime = 0){timer = setTime;}
	

	//process
	const int AddAnim(const EFloatingAnim eFloatingAnim, const ldwPoint sPos);
	const int AddAnim(const EFloatingAnim eFloatingAnim, const ldwPoint sPos, const bool bIsCentered);
	const int AddAnim(const EFloatingAnim eFloatingAnim, const ldwPoint sPos, int iAnimSpeed, const bool bIsLooping, const EFloatingAnimEffect eFloatingAnimEffect, const EScenePriority eScenePriority, const bool bIsCentered = false, const int iEffectSpeed = 1);
	void RemoveType(const EFloatingAnim eFloatingAnim);
	void Remove(const int iAnim);

	void ScareFish(ldwPoint scarePoint);

	void Draw() const;
	void DrawOverlays() const;
	void Draw(const int iFloatingAnim) const;
	void Update();
	
	//access
	void AllowAddWhilePaused(const bool bAllowAdd);
	const int GetNumberOfType(const EFloatingAnim eType) const;
	void SetPos(const int iAnim, const ldwPoint sPos);

	EFloatingAnim GetType(int iAnim);
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CFloatingAnim		FloatingAnim;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/
