/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	LightningBolt
**
** DESCRIPTION:	Manages a number of LightningBolt emitters
**
**      AUTHOR:	Adrian Francis
**
\******************************************************************************/

#ifndef LIGHTNINGBOLT_H
#define LIGHTNINGBOLT_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwTypes.h"
#include "FloatingAnim.h"
#include "../Utils/SceneManager.h"
#include "../Villagers/VillagerAttachmentInterface.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int LightningBolt_c_iMaxEmitters = 128;
const int LightningBolt_c_iMaxSegmentsPerEmitter = 32;
const int LightningBolt_c_iMaxChildrenPerSegment = 2;
const int LightningBolt_c_iNumLeaderFrames = 4;
const int LightningBolt_c_iNumDischargeFrames = 8;

#ifndef _MAC
const float M_PI = 3.141592654f;
#endif

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CLightningBolt : public IVillagerAttachment
{
private:

	//class enumerations
	enum ESegmentType
	{
		eSegmentType_Leader,
		eSegmentType_PrimaryLeader
	};

	//class structures
	struct SSegment
	{
		ldwPoint		m_sStart;
		ldwPoint		m_sEnd;
		int				m_iLength;
		float			m_fStartAlpha;
		float			m_fEndAlpha;

		int				m_iParent;
		int				m_iDepth;
		ESegmentType	m_eType;

		int				m_iChild[LightningBolt_c_iMaxChildrenPerSegment];
		int				m_iNumChildren;
	};

	struct SEmitter
	{
		bool			m_bIsActive;
		ldwPoint		m_sPos;
		int				m_iLength;
		ldwColor		m_sColor;
		int				m_iThickness;
		EScenePriority	m_eScenePriority;
		int				m_iSceneOffset;

		int				m_iFrame;

		SSegment		m_sSegment[LightningBolt_c_iMaxSegmentsPerEmitter];
		int				m_iNumSegments;

		public:		const int AddSegment(const ldwPoint sPos, const int iMaxLength, const float fDirection, const int iParent = -1);
		public:		void UpdateSegment(const int iSegmentIndex, const int iDistanceToGround, const int iDistanceFromSourceToGround, const float fAlphaMultiplier);
	};

	//class data
	SEmitter			m_sEmitter[LightningBolt_c_iMaxEmitters];

	//class methods
	//setup
	public:		CLightningBolt(void);
	public:		void Reset();

	// process
	public:		const int Add(const ldwPoint sPos, const int iLength, const float fDirection, const ldwColor sColor, const int iThickness, const EScenePriority eScenePriority, const int iSceneOffset = 0);
	public:		void Remove(const int iEmitter);

	public:		void Update(void);
	public:		void Draw(void);
	public:		void Draw(const int iEmitter);

	// access
	public:		void SetPos(const int iEmitter, const ldwPoint sPos);
	public:		const bool IsActive(const int iEmitter);
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CLightningBolt		LightningBolt;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/