/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Bath 
**
** DESCRIPTION:	Bath Restoration Puzzle
**
**      AUTHOR:	Arthur Humphrey, from template of Adrian Francis
**
\******************************************************************************/

#ifndef RESEARCHTOTEMPUZZLE_H
#define RESEARCHTOTEMPUZZLE_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwImage.h"
#include "PuzzleInterface.h"
#include "../Game/theGameState.h"
#include "../Game/GodPowerManager.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

//const ldwPoint Fruitwells_c_sCenter(114, 1802);

enum EResearchTotemStatus
{
	eResearchTotem_InitialState = 0,
	eResearchTotem_WorkStarted,
	eResearchTotem_OneQuarterDown,
	eResearchTotem_OneHalfDown,
	eResearchTotem_ThreeQuartersDown,
	eResearchTotem_TotemDown,
	eResearchTotem_NumberOfStates

};

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CResearchTotemPuzzle : public IPuzzle
{
	friend class CPuzzle;

public:

	CResearchTotemPuzzle(void);



	EResearchTotemStatus m_eResearchTotemStatus;

	void Reset();

	const int	LoadState(const unsigned char *pData);
	const int	SaveState(const unsigned char *pData, const int iMaxDataSize) const;
	bool		AdvancePuzzleState();
	EResearchTotemStatus	 GetStatus();
	int			GetPercentComplete();

	void Draw() const;
	

	void		Draw(const int iRef);
	const int	SaveStateSize() {return sizeof (SSaveState);}
	void		LoadAssets();
	void		ReleaseAssets();

	// process
	void Refresh();
	void Update();
	void Advance(CVillager &Villager, const int iRef);
	void Advance(EResearchTotemStatus currentStatus);
	void Complete();

	// access

private:
	

	//class structures
	struct SSaveState
	{
		EResearchTotemStatus	m_eResearchTotemStatus;
		int					m_iResearchTotemProgress;
	};
	
	int m_iResearchTotemProgress;

	//class data
	
	theGameState  *gs;

	ldwImageGrid *	m_pScaffold;

	ldwImageGrid *	m_pTotem;

};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CResearchTotemPuzzle		ResearchTotemPuzzle;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/