/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Bath 
**
** DESCRIPTION:	Bath Restoration Puzzle
**
**      AUTHOR:	Arthur Humphrey, from template of Adrian Francis
**
\******************************************************************************/

#ifndef RestoreMausoleumPUZZLE_H
#define RestoreMausoleumPUZZLE_H
#define InitialDebris 100
#define FirstExtraDebrisThreshold 4
#define SecondExtraDebrisThreshold 8
#define MaxExtraDebrisThreshold 900

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwImage.h"
#include "PuzzleInterface.h"
#include "../Game/theGameState.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

//const ldwPoint Fruitwells_c_sCenter(114, 1802);

enum ERestoreMausoleumStatus
{
	eRestoreMausoleum_InitialState = 0,
	eRestoreMausoleum_WorkStarted,
	eRestoreMausoleum_OneQuarterDown,
	eRestoreMausoleum_OneHalfDown,
	eRestoreMausoleum_ThreeQuartersDown,
	eRestoreMausoleum_TotemDown,
	eRestoreMausoleum_NumberOfStates

};


/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CRestoreMausoleumPuzzle : public IPuzzle
{
	friend class CPuzzle;

public:

	CRestoreMausoleumPuzzle(void);

	ERestoreMausoleumStatus m_eRestoreMausoleumStatus;

	void Reset();

	const int	LoadState(const unsigned char *pData);
	const int	SaveState(const unsigned char *pData, const int iMaxDataSize) const;
	bool		AdvancePuzzleState();
	ERestoreMausoleumStatus	 GetStatus();
	int GetPercetageRubbleRemaining();

	void Draw() const;
	

	void		Draw(const int iRef);
	const int SaveStateSize() {return sizeof (SSaveState);}
	void		LoadAssets();
	void		ReleaseAssets();

	// process
	void Refresh();
	void Update();
	void Advance(CVillager &Villager, const int iRef);
	void Advance();
	void AdvanceHeathen();
	void SetDebrisRemaining(float debrisRemaining);
	void Complete();
	void HandleMausoleumButton();

	void InflictEarthQuakeDamage(void);

	// access

private:
	

	//class structures
	struct SSaveState
	{
		ERestoreMausoleumStatus	m_eRestoreMausoleumStatus;
		float					m_fRestoreMausoleumProgress;
	//	int					m_iPointsToComplete;
	//	int					m_iCurrentMaxProgressPoints;

	};
	
	float m_fRestoreMausoleumProgress;
	int	m_iCurrentMaxProgressPoints;

	ldwUINT32 m_uPlantTimer;
	//class data
	
	theGameState  *gs;

	ldwImageGrid *	m_pScaffold;

	ldwImageGrid *	m_pTotem;

	ldwImageGrid *	m_pRocks;

	ldwImageGrid *	m_pDebrisStrip;

};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CRestoreMausoleumPuzzle		RestoreMausoleumPuzzle;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/