/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	SceneManager
**
** DESCRIPTION:	Manages drawing priority of scene elements
**
**      AUTHOR:	Adrian Francis
**
\******************************************************************************/

#ifndef SCENE_MANAGER_H
#define SCENE_MANAGER_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwTypes.h"
#include "ldwImage.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int SceneManager_c_iMaxElements = 1024;

/******************************************************************************\
** CLASS ENUMERATIONS
\******************************************************************************/
enum ESceneElement
{
	eSceneElement_None,

	eSceneElement_Bird,
	eSceneElement_CollectableItem,
	eSceneElement_Environment,
	eSceneElement_FloatingAnim,
	eSceneElement_GodPower,
	eSceneElement_Graveyard,
	eSceneElement_Puzzle,
	eSceneElement_Villager,
	eSceneElement_LightningBolt
};

enum EScenePriority
{
	eScenePriority_Underwater,
	eScenePriority_Water,
	eScenePriority_OnWater,
	eScenePriority_AboveWater,
	eScenePriority_Ground,
	eScenePriority_OnGround,
	eScenePriority_Background,
	eScenePriority_Normal,
	eScenePriority_Foreground,
	eScenePriority_Flying,
	eScenePriority_InHand,
	eScenePriority_UserInterface
};

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CSceneManager
{
public:
	//class methods
	//setup
    CSceneManager(void);

	//process
	void BeginScene(const ldwPoint sBasePos = ldwPoint(0, 0), const float fScale = 1.0f);
	void EndScene();

	void AddElement(const ESceneElement eType, const int iRef, const EScenePriority eScenePriority, const int iOffset = 0);

	void Draw(ldwImage *pImage, const ldwPoint sPos, const float fAlpha = 1.0f) const;
	void Draw(ldwImageGrid *pImage, const ldwPoint sPos, const int iFrame, const float fAlpha = 1.0f) const;

	void DrawTinted(ldwImage *pImage, const ldwPoint sPos,ldwColor color, const float fAlpha = 1.0f) const;
	void DrawTinted(ldwImageGrid *pImage, const ldwPoint sPos, const int iFrame,ldwColor colork, const float fAlpha = 1.0f, float fScale = 1.0f) const;
	void DrawTinted(ldwImageGrid *pImage, const ldwPoint sPos, const int iRow, const int iColumn,ldwColor color, const float fAlpha = 1.0f, float fScale = 1.0f) const;

	void DrawScaled(ldwImage *pImage, const ldwPoint sPos, const float fScale, float alpha = 1.0f,const bool bScaleRelative = false) const;
	void DrawScaled(ldwImageGrid *pImage, const ldwPoint sPos, const int iRow, const int iColumn, const float fScale,float fAlpha = 1.0f) const;
	void DrawScaled(ldwImageGrid *pImage, const ldwPoint sPos, const int iCell, const float fScale,float fAlpha = 1.0f) const;

	//access
	const ldwPoint BasePos() const { return m_sBasePos; }
	const float Scale() const { return m_fScale; }

private:

	//class structures
	struct SElement
	{
		ESceneElement	m_eType;
		int				m_iRef;
		int				m_iSortIndex;
	};

	//class data
	SElement			m_sElement[SceneManager_c_iMaxElements];
	int					m_iNumElements;

	ldwPoint			m_sBasePos;
	float				m_fScale;
	int					m_iScale;

};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CSceneManager		SceneManager;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/