/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Villager
**
** DESCRIPTION:	Represents an individual villager
**
**
\******************************************************************************/

#ifndef VILLAGER_H
#define VILLAGER_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include <string.h>
#include "../Game/Isola.h"
#include "ldwTypes.h"
#include "../Sound/Sound.h"
#include "AnimControl.h"
#include "BehaviorTypes.h"
#include "LikeList.h"
#include "DislikeList.h"
#include "VillagerSkills.h"
#include "VillagerBio.h"
#include "VillagerState.h"
#include "VillagerAI.h"
#include "VillagerAttachmentInterface.h"
#include "../Utils/SceneManager.h"
#include "../World/ContentMap.h"
#include "../World/WorldPath.h"

class theGameState;
class ldwGameWindow;
class theStringManager;
class ldwLog;
class thePrizeMsgAssembler;

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int Villager_c_iWidth = 40;
const int Villager_c_iHeight = 65;
const int Villager_c_iMaxBehaviourLabelLength = 40;
const int Villager_c_iMaxAttachments = 12;
const int Villager_c_iChanceOfRandomGenes = 5;
const int Villager_c_iGravity = 3;
const int Villager_c_iMaxHistory = 10;

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CVillager : public CVillagerAI, public CVillagerBio, public CVillagerState, public CVillagerSkills, public CAnimControl
{
	friend class CVillagerAI;
	friend class CVillagerPlans;
	friend class CVillagerManager;

public:
	// class enums

	enum EHeathenRoles
	{
		eHeathenRole_None = 0,
		eHeathenRole_Food1,
		eHeathenRole_Research,
		eHeathenRole_Food2,
		eHeathenRole_Medicine,
		eHeathenRole_Bathing,
		eHeathenRole_Builder,
		eHeathenRole_WoodGuards,
		eHeathenRole_Food1TotemGuard,
		eHeathenRole_ResearchTotemGuard,
		eHeathenRole_LabGuard,
		eHeathenRole_BerryGuard,
		eHeathenRole_LeadDoctor,
		eHeathenRole_Chief,
		eHeathenRole_LeadResearcher,
		eHeathenRole_LeadBuilder,
		eHeathenRole_LeadFarmer,
		eHeathenRole_Mommy,
		eHeathenRole_DryLake,
		eHeathenRole_NumberOf
	};
	//class structures
	struct SSaveState
	{
		bool				m_bExists;
		bool				m_bIsHeathen;
		bool				m_bIsAggressive;
		bool				m_bIsScary;
		bool				m_bIsCurious;
		int					m_iFaith;
		EHeathenRoles		m_eHeathenRoles;

		CVillagerBio		m_Bio;
		CVillagerState		m_State;
		CVillagerSkills		m_Skills;
		CLikeList			m_Likes;
		CDislikeList		m_Dislikes;

		ldwINT32			m_evangelizedTimer;
	};

	struct SAttachment
	{
		IVillagerAttachment *	m_pAttachment;
		int						m_iRef;
		EFloatingAnim           m_floatingAnim;
		ldwPoint				m_sOffset;
		unsigned int			m_uiTimeOut;
		unsigned int			m_uiAttachmentTime;
	};
	
	struct SHistory
	{
		ldwPoint				m_sPos;
		EAnimFrame				m_eAnimFrame;
		EDirection				m_eDirection;
		EHeadDirection			m_eHeadDirection;
	};

	//class methods
	//setup

	CVillager(void);
	void Reset(void);
	void Init(int lName, ESkill whichSkill, int howMuchSkill, EGender whatGender, int whatAge, const char * fatherName, const char * motherName, int fatherBodyType, int fatherHeadType, int motherBodyType, int motherHeadType, int forceX, int forceY, bool oldClothes);
	void InitTwin(CVillager &Twin);
	const bool SaveState(SSaveState &sSaveState);
	const bool LoadState(SSaveState &sSaveState);

	// process

	void DrawDetailVillager();
	void DrawDetailVillager(ldwPoint feetPos);
	void DrawDetailVillagerParents();
	void DrawEventVillager(int x, int y,EBodyPosition bodyPosition = eBodyPosition_Upright,float scale = 1.0f,bool drawShadow = true);

	void Update();

	void NewBehavior(const EBehavior eBehavior, const SBehaviorData &sData = SBehaviorData());
//	void StartNewBehavior() { CVillagerAI::StartNewBehavior(/**this*/); }
//	void ForgetPlans() { CVillagerAI::ForgetPlans(/**this*/); }
//	void NextPlan() { CVillagerAI::NextPlan(/**this*/); }

	void ConceiveChildWith(CVillager *pPartner, bool gameInitPreg = false);
	void Impregnate(int lastName, ESkill fatherSkillType, int fatherSkillValue, const char * fatherName, int fatherHeadType, int fatherBodyType, bool gameInitPreg = false);
	void DropVillager();
	void NormVel(int speed);
	void Stop();

	void SetVillagerInHand();

	const int ChanceOfDisease();

	void AddLike(ELike eLike) { m_Likes.Add(eLike); m_Dislikes.Remove(eLike); }
	void RemoveLike(ELike eLike) { m_Likes.Remove(eLike); }
	void AddDislike(ELike eDislike) { m_Dislikes.Add(eDislike); m_Likes.Remove(eDislike); }
	void RemoveDislike(ELike eDislike) { m_Dislikes.Remove(eDislike); }

	const bool Attach(IVillagerAttachment &Attachment, const int iRef, int iTime = 0) { return Attach(Attachment, iRef, ldwPoint(0, 0), iTime); }
	const bool Attach(IVillagerAttachment &Attachment, const int iRef, const ldwPoint sOffset, const int iTime = 0);
	void Detach(IVillagerAttachment &Attachment, int &iRef);
	void DetachAll();
	void RefreshAttachments();

	// access
	void       SetExists(bool bExists) { m_bExists = bExists; }
	const bool Exists(bool includeGhosts = false) const;

	void       SetIndex(int iIndex) { m_iIndex = iIndex; }
	const int  Index() const        { return m_iIndex; }

	void       SetFocus(bool bHasFocus);
	const bool HasFocus() const;

	void       SetInHand(bool bIsInHand) { m_bIsInHand = bIsInHand; }
	const bool IsInHand() const          { return m_bIsInHand; }

	void       SetBusy(bool bIsBusy) { m_bIsBusy = bIsBusy; }
	const bool IsBusy() const        { return m_bIsBusy; }

	void       SetIsGhost(const bool bIsGhost, const ECarrying nearbyCollectable = eCarrying_Nothing, const float fAlpha = 0.6f) { m_bIsGhost = bIsGhost; m_ghostAlpha = fAlpha; m_GhostCollectable = nearbyCollectable; }
	const bool IsGhost() const                                       { return m_bIsGhost; }

	void			   SetHeathenRole(EHeathenRoles role) {m_eHeathenRoles = role;}
	EHeathenRoles	   GetHeathenRole() {return m_eHeathenRoles;}

	void SetIsHeathen(bool bIsHeathen = true);
	void SetAggressive(bool bIsAggressive)  { m_bIsAggressive = bIsAggressive ;}
	void SetScary(bool bIsScary)  { m_bIsScary = bIsScary; }
	void SetCurious(bool bIsCurious)  { m_bIsCurious = bIsCurious; }

	ldwUINT32 GetEvangelismTimer() { return m_evangelizedTimer; } 
	void SetEvangelismTimer(ldwINT32 timerSetting) { m_evangelizedTimer = timerSetting; } 

	void SetEvangelismAnimFrame(bool frame)  { m_evangelizedAnimTickCounter = frame; }
	void IncEvangelismAnimFrame()  { m_evangelizedAnimTickCounter++; }
	int GetEvangelismAnimFrame() const { return m_evangelizedAnimTickCounter; }

	ldwINT32 GetColorEffectTimer() { return m_lColorEffectTimer; } 
	void SetColorEffectTimer(ldwINT32 timerSetting) { m_lColorEffectTimer = timerSetting; } 
	bool ColorEffectIsOn() { return m_bColorEffect; } 
	void SetColorEffect(bool setting); 
	ldwColor GetColorEffectColor() { return m_lColorEffectColor; }
	void SetColorEffectColor(int red, int green, int blue, int alpha) { m_lColorEffectColor.R = red; m_lColorEffectColor.G = green; m_lColorEffectColor.B = blue; m_lColorEffectColor.A = alpha; }

	void SetFaith(int iFaith, const bool bAllowConversion = true); 
	void AdjFaith(const int iAmount); 

	void FaithAttrition();

	const bool IsHeathen() const { return m_bIsHeathen; }
	const bool IsAggressive() const { return m_bIsAggressive; }
	const bool IsScary() const { return m_bIsScary; }
	const bool IsCurious() const { return m_bIsCurious; }

	const int Faith() const { return m_iFaith; }

	const bool Likes(ELike eLike) { return m_Likes.Contains(eLike); }
	const bool Dislikes(ELike eDislike) { return m_Dislikes.Contains(eDislike); }
	const char *GetLikesAsString() { return m_Likes.GetListAsString(); }
	const char *GetDislikesAsString() { return m_Dislikes.GetListAsString(); }

	const EBehavior CurrentBehavior() const { return m_eBehavior; }
	void SetBehavior(EBehavior whatBehavior) { m_eBehavior = whatBehavior; }
	void SetBehaviorLabel(char *label) { strncpy(m_sBehaviorLabel, label, Villager_c_iMaxBehaviourLabelLength - 1); }
	void SetBehaviorLabel(StringId eString) { strncpy(m_sBehaviorLabel, theStringManager::Get()->GetString(eString), Villager_c_iMaxBehaviourLabelLength - 1); }
	const char *BehaviorLabel() const { return m_sBehaviorLabel; }

	const float Scale() const;
	const ldwPoint Dimensions() const;

	void SetXPos(int iXPos);
	const int XPos() const { return m_iXPos; }
	void SetYPos(int iYPos);
	const int YPos() const { return m_iYPos; }
	void SetPos(const ldwPoint sPos);
	const ldwPoint Pos() const { return ldwPoint(m_iXPos, m_iYPos); }
	void SetFeetPos(const ldwPoint sPos);
	const ldwPoint FeetPos() const;
	const bool IsWithinRangeOf(const CContentMap::EObject eObject, const int iRange) const;
	const bool IsWithinRangeOf(const CVillager *pVillager, const int iRange) const;
	const EAnimFrame AnimFrame() const { return m_eAnimFrame; }
	const int DisplacementFrame() const { return m_iDisplacementFrame; }
	void ResetDisplacementFrame() { m_iDisplacementFrame = 0; }
	void SetScenePriority(const EScenePriority eScenePriority) { m_eScenePriority = eScenePriority; }
	const EScenePriority ScenePriority() { return m_eScenePriority; }
	void SetSpeedMultiplier(const int iSpeedMultiplier) { m_iSpeedMultiplier = iSpeedMultiplier; }
	const int SpeedMultiplier() const { return m_iSpeedMultiplier; }
	void SetScaleMultiplier(const float fScaleMultiplier) { m_fScaleMultiplier = fScaleMultiplier; }
	const float ScaleMultiplier() const { return m_fScaleMultiplier; }
	void SetAlphaMultiplier(const float fAlphaMultiplier) { m_fAlphaMultiplier = fAlphaMultiplier; }
	const float AlphaMultiplier() const { return m_fAlphaMultiplier; }

	const EDirection Direction() const { return m_eDirection; }
	void SetHeadDirection(const EHeadDirection eDirection) { if (AnimManager.IsValidHeadDirection(eDirection, m_eDirection)) m_eHeadDirection = eDirection; }
	const EHeadDirection HeadDirection() const { return m_eHeadDirection; }
	const EBodyPosition BodyPosition() const { return m_eBodyPosition; }


	void SetCarrying(ECarrying eCarrying);
	void SetCarryingBabyAndUpdateTrophies(void);
	void ForceCarrying(ECarrying eCarrying);
	const ECarrying Carrying() const { return m_eCarrying; }

	void RecordHistory(void);
	void RecallHistory(const int iOffset);
	const int HistoryCount(void) const { return m_iHistoryCount; }

	const bool HasAttachment(IVillagerAttachment &Attachment, int iRef) const;

	bool IsCleanOfMind();
	bool IsCleanOfBody();
	bool HasBees();
	bool IsSappy();

	void SayOh();

	ETerritory  LegalTerritory() {return m_legalTerritory;}

protected:

	void StartBending();
	void StartCheering();
	void StartTwirlingCW();
	void StartTwirlingCCW();
	void StartDiscoing();
	void StartDoingTheMonkey();
	void StartReveling();
	void StartLyingDown();
	void StartScrubbing();
	void StartExorcisting();
	void StartJoyTwirlingCW();
	void StartClimbing(const SClimbData sData);
	void StartDancing();
	void StartDiving(SDiveData &sData);
	void StartEmbrace();
	void StartGoing(SGoingData &sData);
	void StartHiding();
	void StartHealing();
	void StartEvangelizing();
	void StartJumping(SJumpData &sData);
	void StartNothing();
	void StartSwimming(const SSwimData sData);
	void StartWaiting(const EBodyPosition eBodyPosition, const EDirection eDirection, const EHeadDirection eHeadDirection);
	void StartWandering(SWanderingData &sData);
	void StartFollowVillager(SFollowVillagerData &sData);
	void StartWorking();
	void StartDoingPushups();

	void ProcessClimbing(const SClimbData sData);
	void ProcessDancing();
	void ProcessDiving(SDiveData &sData);
	void ProcessEmbrace();
	void ProcessFollowPath(SGoingData &sPlan);
	void ProcessGoing(SGoingData &sData);
	void ProcessGoingRelative(SGoingData &sData);
	void ProcessJumping(SJumpData sData);
	void ProcessSwimming();
	void ProcessWaiting();
	void ProcessMeditating();
	void ProcessWandering(SWanderingData &sData);
	void ProcessFollowVillager(SFollowVillagerData &sData);

	void AdjustXPos(int iAdj) { m_iXPos += iAdj; }
	void AdjustYPos(int iAdj) { m_iYPos += iAdj; }
	void SetXVel(int iXVel) { m_iXVel = iXVel; }
	void SetYVel(int iYVel) { m_iYVel = iYVel; }
	void SetBodyPosition(const EBodyPosition eBodyPosition) { m_eBodyPosition = eBodyPosition; }
	void ResetToNeutralBodyPosition();

	CVillager* IsAnyoneNearby(int howClose);

private:

	void AddFootprints();
	void AddDust();

	void SelectDirectionBasedOnVelocity();

	void ConvertToBeliever();
	void ConvertToHeathen();

	//class data
	int						m_iIndex;		// index of villager in VillagerManager::villager[]	array

	int						m_iXPos;
	int						m_iYPos;
	int						m_iXVel;
	int						m_iYVel;
	int						m_iXMom;
	int						m_iYMom;
	int						m_iFrameIndex;
	EAnimFrame				m_eAnimFrame;
	int						m_iAnimDirection; //1 or -1 depending if bending down or up, for example
	int						m_iDisplacementFrame;
	int						m_iDisplacementDirection;
	EScenePriority			m_eScenePriority;
	int						m_iSpeedMultiplier;
	float					m_fScaleMultiplier;
	float					m_fAlphaMultiplier;

	bool					m_bExists;
	bool					m_bIsInHand;
	bool					m_bIsBusy;

	ldwINT32				m_lColorEffectTimer;
	bool					m_bColorEffect;
	ldwColor				m_lColorEffectColor;

	bool					m_bIsGhost;
	ECarrying				m_GhostCollectable;
	float					m_ghostAlpha;

	bool					m_bIsHeathen;
	bool					m_bIsAggressive;
	bool					m_bIsScary;
	bool					m_bIsCurious;

	int						m_iFaith;

	ldwINT32				m_evangelizedTimer;
	int						m_evangelizedAnimTickCounter;

	EHeathenRoles			m_eHeathenRoles;

	EDirection				m_eDirection;
	EHeadDirection			m_eHeadDirection;
	EBodyPosition			m_eBodyPosition;
	ECarrying				m_eCarrying;
	
	EBehavior				m_eBehavior;
	char					m_sBehaviorLabel[Villager_c_iMaxBehaviourLabelLength];

	SAttachment				m_sAttachment[Villager_c_iMaxAttachments];
	int						m_iNumAttachments;

	SHistory				m_sHistory[Villager_c_iMaxHistory];
	int						m_iHistoryCount;

	CLikeList				m_Likes;
	CDislikeList			m_Dislikes;

	CWorldPath				mNavPath;	// not persisted in gamestate

	theGameState *			gs;
	ldwGameWindow *			gameWindow;
	theStringManager *		stringMgr;
	ldwLog *				log;
	thePrizeMsgAssembler *	PrzMsg;

	EHeadDirection			m_eDetailHeadDirection;

	ETerritory				m_legalTerritory;
};

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/