/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	VillagerAI
**
** DESCRIPTION:	Villager AI
**
**
\******************************************************************************/

#ifndef VILLAGER_AI_H
#define VILLAGER_AI_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "BehaviorTypes.h"
#include "VillagerSkills.h"
#include "VillagerPlans.h"

class CVillager;

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CVillagerAI : public CVillagerPlans
{
	friend class CVillagerPlans;

	//class methods
	//setup
public:

	CVillagerAI(CVillager *baseVillager) : CVillagerPlans(baseVillager), m_villager(*baseVillager) {}

	void Reset();

	//process
	void Update();

	const bool DecideOnChildBehavior();
	const ESkill DecideOnWorkSkill();

	const bool AttemptSkill(const ESkill eSkill, const SSkillData sData = SSkillData());
	const bool InitiateWorkPlan(ESkill whatSkill);

	void RealtimeWorkDone();

	void DoBevMacInstantly();
	
	void AcquireTarget(const int iTargetX, const int iTargetY, const ESpeed eSpeed);
	void TakePosition();

	bool MaybeInteract(CVillager &nearbyVillager);

protected:

	void AdvanceState(int whichState);

private:

	void 	   DecideWhatToDo();
	const bool DecideOnIdleBehavior();
	const bool DecideOnPregnantBehavior();
	const bool DecideOnStormyBehavior();			
	const bool DecideOnSickBehavior();
	const bool DecideOnExoticBehavior();
	const bool DecideOnEnlightenment();
	const bool DecideOnCuriousBehavior();
	const bool DecideOnRecreation();
	const bool DecideOnDayToDayBehavior();

	void DecideWhatToDoHeathen();
	void HeathenDecideOnNonRoleBehavior();

	void SetRandomHeathenRole();


	
	const bool InitiatePredisposition();

	//class data
	ldwUINT32				m_uiThinkTimer;
	CVillager&				m_villager;
};

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/