/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	VillagerManager
**
** DESCRIPTION:	Manages the entire group of villagers
**
**
\******************************************************************************/

#ifndef VILLAGER_MANAGER_H
#define VILLAGER_MANAGER_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "../Game/Isola.h"
#include "Villager.h"
//#include "Game/theGameState.h"
#include "ldwGameWindow.h"
#include "ldwTypes.h"
#include "../Effects/FloatingAnim.h"
#include "../Game/CelebrationEvent.h"
#include "../Tools/DebuggerInterface.h"
#include "../Utils/theStringManager.h"

class thePrizeMsgAssembler;
class ldwSoundTrack;
class ldwSoundFx;
class theGameState;
class ldwGameWindow;
class ldwLog;

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int VillagerManager_c_iMaxVillagers = VV4_MAX_NUM_VILLAGERS;	// VV4_MAX_NUM_VILLAGERS defined in Isola.h so VillagerManagers.h
																	//  isn't included in theGameState.h - this avoids massive recompile when this file changed
#define researchTableX							(821 + randN(180))
#define researchTableY							(630 + randN(3))

#define numTotems								8

#define reaquisitionFreq						10 //how often to reacquire a target (in iterations)
#define targetTolerance							2	//within how many pixels is considered "at target"

const int cHeathenChasingRadius = 100;

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CVillagerManager : IDebugger
{
public:

	// Made these public for now (Adrian 08/18/07)
	ldwImage *villagerShadow;

	ldwImageGrid *femaleBodies;
	ldwImageGrid *maleBodies;
	ldwImageGrid *femaleActions;
	ldwImageGrid *maleActions;
	ldwImageGrid *femaleHeads;
	ldwImageGrid *maleHeads;
//	ldwImageGrid *femaleHeadsOld;
//	ldwImageGrid *maleHeadsOld;

	ldwImage *focusCircle;
	ldwImageGrid *carriedItems;
	ldwImageGrid *		m_pCollectables;
	ldwImageGrid *skelly;
	ldwImageGrid *ripple;
	ldwImageGrid *totems;
	ldwImageGrid *evangelizedSparkle;

	
	//class methods
	//setup
	CVillagerManager();
	void Reset();
	void LoadAssets();
	void ReleaseAssets();
	const bool LoadState(CVillager::SSaveState &sSaveState);
	const bool SaveState(CVillager::SSaveState &sSaveState);

	//process
	void Debug();

	void InitStartingPositions();
	void TakePositions(bool heathensOnly = false);

	void RefreshAttachments();

	const int SpawnRandomPeep(bool IsHeathen);

	const int SpawnSpecificPeep(const int lName, const ESkill whichSkill, const int howMuchSkill, const EGender whatGender, const int whatAge, const char * fatherName, const char * motherName, const int fatherBodyType, const int fatherHeadType, const int motherBodyType, const int motherHeadType, const int forceX, int const forceY, const bool oldClothes = false);
	const int SpawnSpecificPeep(const int lName, const ESkill whichSkill, const int howMuchSkill, const EGender whatGender, const int whatAge, const char * fatherName, const char * motherName, const int fatherBodyType, const int fatherHeadType, const int motherBodyType, const int motherHeadType);
	const int SpawnSpecificPeep(const int lName, const ESkill whichSkill, const int howMuchSkill, const EGender whatGender, const int whatAge);
	const int SpawnSpecificPeep(CVillager *twinOfWhichVillager);
	const int SpawnSpecificPeep(CVillager &twinOfWhichVillager);
	const int SpawnSpecificHeathen(const int lName, const ESkill whichSkill, const int howMuchSkill, const EGender whatGender, const int whatAge, const char * fatherName, const char * motherName, const int fatherBodyType, const int fatherHeadType, const int motherBodyType, const int motherHeadType, const int forceX, const int forceY, const bool oldClothes, bool isAggressive, bool isScary, CVillager::EHeathenRoles role = CVillager::eHeathenRole_None, int faith = -55);
	const int SpawnSpecificHeathen(const int lName, const ESkill whichSkill, const int howMuchSkill, const EGender whatGender, const int whatAge, bool isAggressive = false, bool isScary = false, CVillager::EHeathenRoles role = CVillager::eHeathenRole_None, int faith = -55);
	const int GetNumChildren(void);
	const int CVillagerManager::CountGirls();
	const int CVillagerManager::CountBoys();
	const int SpawnGhost(const EGender whatGender,const int whatAge,const int bodyType,const int headType,ldwPoint pos,ECarrying collectable);
	const int SpawnSkelly(const ldwPoint sFeetPos, const int iAge);

	void IterateAllVillagers();
	void AllVillagersTimeflowUpkeep();	// TODO: Need a better name!
	void AllVillagersRealtimePhysiologyAndProductivityUpkeep();	// TODO: Need a better name!
	void DrawVillagers();
	void DrawVillager(const int iVillager,float alpha = 1.0f);

	bool IsScaryGuardAtMausoleum();

	CVillager *GetMatchingVillager(int minAge = agesAnyAge, int maxAge = agesAnyAge, EGender gender = eGender_Unknown, bool pregOK = true, bool sickOK = true, ESkill requiredSkill = eSkill_None, int skillAmount = 0, bool pregREQ = false, bool sickREQ = false, int *candidateCount = 0,bool maskedREQ = false, bool carryingOK = true, bool heathenOK = false);
	CVillager *ImpregnateMatchingVillagers(int maxAge = agesAnyAge, int numPreg = 1, bool sickOK = true, ESkill requiredSkill = eSkill_None, int skillAmount = 0, int *candidateCount = 0);
	CVillager *GetVillagerWithPredisposition(int preDisposition, int *candidateCount = 0);

	const int SelectOtherAvailableMatingVillager(CVillager &villagerToOmit);
	const int SelectPlayMateFor(CVillager &Villager);
	const int SelectOtherSickVillager(CVillager &VillagerToOmit);
	CVillager *CheckForOtherWaitingVillagerAtPoint(const CVillager *pWhoIsChecking, const ldwPoint click);
	void MakeAllVillagersDoIt(EBehavior eBehavior, int minAge = agesAnyAge, int maxAge = agesAnyAge, EGender gender = eGender_Unknown, int *candidatecount = 0, int minCandidates = 0);
	void MakeAllHeathensDoIt(EBehavior eBehavior, int minAge = agesAnyAge, int maxAge = agesAnyAge, EGender gender = eGender_Unknown, int *candidatecount = 0, int minCandidates = 0);

	void MakeSomeVillagersDoIt(EBehavior eBehavior,int chance = 50, int minAge = agesAnyAge, int maxAge = agesAnyAge, EGender gender = eGender_Unknown, int *candidatecount = 0, int minCandidates = 0, int maxCandidates = 0);
	void GiveSelectedVillagersSkill(const ESkill skill = eSkill_Farming, int amt = 5, int minAge = agesAnyAge, int maxAge = agesAnyAge, EGender gender = eGender_Unknown, int *candidateCount = 0);
	void MakeSomeVillagersDo(const EBehavior eBehavior, int chance, int minAge = agesAnyAge, int maxAge = agesAnyAge, EGender gender = eGender_Unknown);
	void TeleportAllVillagers(ldwPoint where, int width = 150, int height = 150, int minAge = agesAnyAge, int maxAge = agesAnyAge, EGender gender = eGender_Unknown, int *candidateCount = 0);
	void TeleportWaterBabiesToSafety(const CContentMap::EObject eSafeObject);

	void SetStartingHeathenFaith();

	void MakeRandomVillagersSick(const int chance, const bool injury = false, const int injuryAmt = 0);
	void MakeRandomVillagersDead(const int chance, EGender gender);
	void MakeRandomVillagersVanish(const int chance, EGender gender);
	void CureAllVillagers();
	void MakeAllVillagersForgetWhatToDo(bool forgetPlans);
	void MakeRandomVillagersInjured(const int chance, const int baseinjuryAmt, const int randomAmt);
	void ImproveRandomVillagersSkill(const int chance, const ESkill whichSkill, const int baseAmt, const int randomBonus, const bool needChild);
	void ChangeRandomVillagersClothes(const int chance, const bool needChild);
	void InvokeCelebration(const ECelebrationEvent eCelebrationEvent, const ldwPoint sPos = ldwPoint(0, 0));
	void HealALlVillagers();


	void AddRandomVillagerLike(const int chance, ELike eLike);
	void AddRandomVillagerDislike(const int chance, ELike eDislike);
	void RandomSkillPenalty(CVillager *pVillager, int baseAmount, int randomAmount);
	void RandomSkillBonus(CVillager *pVillager, int baseAmount, int randomAmount);

	const int CheckForVillagerClick(const ldwPoint sClickPos);
	
	void SetTargetVillager(const int iVillagerIndex);
	const int GetTargetVillager(void);

	void MakeInFocus(const int whichVillager);
	void MakeInFocus(const CVillager *pVillager);
	void SetNoFocus();	
	const ldwPoint GetVillagerPosition(const int whichVillager) { return ldwPoint(villager[whichVillager].XPos(), villager[whichVillager].YPos()); }
	
	const int FindCorpse();

	const EGender ExtremeLimitGenderCorrect();

	void GatherChildrenForStory();
	void GatherAtNurseryForInstruction();

	//access
	CVillager *GetVillager(const int iVillager);
	CVillager *GetVillagerInFocus();
	CVillager *GetVillagerAt(const ldwPoint sPos, const int iExceptFor, const bool bIncludeCorpses = false);
	CVillager *GetVillagerWaitingAt(const ldwPoint sPos, const CVillager *pWhoIsChecking);
	CVillager *GetHeathenWithRole(const CVillager::EHeathenRoles eHeathenRole);

	const bool IsRoomToPopulate();
	const int  Population();
	//const int  Totems();
	const int  GenderCounts(int &iMaleCount, int &iFemaleCount);
	const int  MothersCaringForBabies();
	const bool HasAnyVillagerScaredBirds();
	const bool IsAnyoneCarrying(const ECarrying eCarrying) const;

	CVillager *GetVillagerDoing(const EBehavior eBehavior, int *candidateCount = 0);
//	CVillager *GetVillagerDoing(const EBehavior eBehavior);

	void 	ChangeAllBehavior(const EBehavior targetBehavior, const EBehavior newBehavior);
	
	bool VillagerExists(const int iVillager);


	bool DroppedNearMaskedPeeps(CVillager &villager/*, bool onlyAggressive*/);

	void AddToWatchList(int villagerIndex);
	void RemoveFromWatchList(int villagerIndex);

	void SetAlpha(const float fAlpha);

 private:
 	
 	void DrawCarriedItems(const CVillager &Villager, const ldwPoint sPos, const float fAlpha);

	//class data
	ldwGameWindow *gameWindow;

	ldwLog			*log;
	theStringManager *stringMgr;

	CVillager villager[VillagerManager_c_iMaxVillagers];

	int		m_iTargetVillager;

	int	 mWatchedVillagerCount;
	int  mVillagerWatchList[VillagerManager_c_iMaxVillagers];

	bool mVillagerWatched[VillagerManager_c_iMaxVillagers];

	float	m_fAlpha;
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CVillagerManager		VillagerManager;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/