/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	WaypointEditor
**
** DESCRIPTION:	Editor for positioning waypoints
**
**      AUTHOR:	Adrian Francis
**
\******************************************************************************/

#ifndef WAYPOINT_EDITOR_H
#define WAYPOINT_EDITOR_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwTypes.h"
#include "ldwImage.h"
#include "EditorInterface.h"
#include "../World/Waypoint.h"

#include "../World/WorldPath.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CWaypointEditor : public IEditor
{

public:
	//class methods
	//setup
	CWaypointEditor(void);
	
	void Reset();
	void LoadAssets();
	void ReleaseAssets();

	// process
	void Draw();
	void DrawWaypoints();

	const bool HandleKeyCharacter(const char cKey);
	const bool HandleKeyDown(const int iKeyCode);
	const bool HandleKeyUp(const int iKeyCode);
	const bool HandleMouseDown(const ldwPoint sMousePos);
	const bool HandleMouseUp(const ldwPoint sMousePos);
	const bool HandleMouseMove(const ldwPoint sMousePos);

	// access
	const char *GetWaypointName(const EWaypoint eWaypoint) const;
	const char *GetWaypointEnumName(const EWaypoint eWaypoint) const;

private:

	void ConstrainWorldView();

	//class enumerations

	//class data
	EWaypoint		m_eCurrentWaypoint;

	ldwImage *		m_pFocusCircle;

	bool 			m_havePaths;

	ldwPoint		m_dragPos;
	bool			m_draggingMap;
	bool			m_drawPathGradient;

	CWorldPath		m_path1;
	CWorldPath		m_path2;
	CWorldPath		m_path3;
	CWorldPath		m_path4;
	CWorldPath		m_path5;
	CWorldPath		m_path6;
	CWorldPath		m_path7;
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CWaypointEditor		WaypointEditor;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/