//
//	Class definition for the Game State
//
//	- uses ldwGameState as base class
//

#ifndef THE_MINI_BUTTERFLY_CLASS_H
#define THE_MINI_BUTTERFLY_CLASS_H
#include "../Game/theGameState.h"
#include "ldwGameWindow.h"
#include "ldwTypes.h"
#include "../Utils/theGraphicsManager.h"
#include "../World/WorldView.h"

class ldwGameWindow;
class theVillagerClass;
class theFloatingAnims;

#define maxNumMiniButterflies 9
#define defaultMiniButterflyAnimSpeed	2
#define numMiniButterflySets			5

typedef enum
{
	eMBChasing,
	eMBLanding,
	eMBSunning,
	eMBFluttering,
	eMBFollowingVillager,
	eMBPolinating,
	eMBNotExisting

}	MBButterflyStateType;

typedef struct
{
	bool exists;
	int xPos;
	int yPos;
	int xTarget;
	int yTarget;
	int landingTargetX;
	int landingTargetY;
	int xVel;
	int yVel;
	int type;
	int wingPosition;
	int wingMotionIdx;
	int scale;
	MBButterflyStateType butterflyState;

} MBButterflyType;

class CMiniButterflyClass 
{
public:

	static CMiniButterflyClass * Get();
	static void Destroy();
	CMiniButterflyClass();
	~CMiniButterflyClass();

	ldwImageGrid *butterflyGrid;
	void NormVel(int whichButterfly);
	void AcquireTarget(int whichButterfly, int x, int y);
	void InitButterflies();
	void SpawnButterflies(int howMany = 9);
	void SetGroupTarget(int x, int y);
	void SetGroupDefaultTarget(int x, int y);
	bool CheckNearButterflies(ldwPoint checkPoint);
	void ChooseNewBehavior();
	MBButterflyStateType GetState(){return butterflies[0].butterflyState;}
	void Update();
	void Draw();
	void ChangeState(MBButterflyStateType whatState);

protected:

	MBButterflyType butterflies[maxNumMiniButterflies];
	static CMiniButterflyClass *mInstance;

	ldwPoint groupTarget;
	ldwPoint groupDefaultTarget;
	ldwPoint groupRainTarget;

	int groupIndexCounter;
	int groupIndexTrigger;


	ldwGameWindow *gameWindow;
	theGameState *gs;
	theVillagerClass *villagers;
	theFloatingAnims *floatingAnim;

};


/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/

//extern CMiniButterflyClass		MiniButterflies;
#endif
